#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

import rosdistro
from auto_abi_checker.utils import error, comma_list_to_array
from os import chdir, environ
from auto_abi_checker.srcs_apt import SrcAptBase

from os.path import dirname, realpath, join


class SrcROSBase(SrcAptBase):
    def __init__(self, name, ros_distro):
        SrcAptBase.__init__(self, name)
        self.ros_distro = self.detect_ros_distribution(ros_distro)
        self.rosdistro_index = rosdistro.get_index(rosdistro.get_index_url())
        self.cache = rosdistro.get_distribution_cache(self.rosdistro_index,
                                                      self.ros_distro)
        self.distro_file = self.cache.distribution_file
        # More logic could be needed with new ros distributions
        # ROS1 - https://www.ros.org/reps/rep-0003.html
        # ROS2 - http://design.ros2.org/articles/changes.html
        if self.ros_distro == 'melodic':
            self.compilation_flags.append('--std=c++14')
        else:
            self.compilation_flags.append('--std=c++17')
            # needed for gazebo_ros_pkgs
            self.compilation_flags.append('-DBOOST_HAS_PTHREADS=1')
        # Needs to add /opt/ros includes to compile ROS software
        self.compilation_flags.append('-I' +
                                    join('/opt/ros/', self.ros_distro, 'include'))

    def detect_ros_distribution(self, user_ros_distro):
        if user_ros_distro:
            return user_ros_distro
        try:
            if environ['ROS_DISTRO']:
                return environ['ROS_DISTRO']
            error("Not ROS distribution provided or ROS_DISTRO environment var")
        except KeyError:
            error("ROS_DISTRO environment variable not found")

    def get_debian_package_name_prefix(self):
        return 'ros-%s-' % self.ros_distro

    def get_debian_ros_package_name(self, ros_package_name):
        return '%s%s' % \
            (self.get_debian_package_name_prefix(),
             ros_package_name.replace('_', '-'))


class SrcROSRepoGenerator(SrcROSBase):
    def __init__(self, name, ros_distro=''):
        SrcROSBase.__init__(self, name, ros_distro)

    def validate_repo(self, ros_repo):
        keys = self.distro_file.repositories.keys()
        if ros_repo in keys:
            return True

        return False

    def get_release_repo(self, ros_repo):
        return self.distro_file.repositories[ros_repo].release_repository

    def validate(self, ros_repo):
        # Check that repo exists in ROS
        if (not self.validate_repo(ros_repo)):
            error("ROS repository " + ros_repo +
                  " not found in the rosdistro index")

    def get_deb_package_names(self, ros_repo):
        ros_pkgs = self.get_release_repo(ros_repo).package_names
        return [self.get_debian_ros_package_name(p)
                for p in ros_pkgs]


class SrcROSPkgGenerator(SrcROSBase):
    def __init__(self, name, ros_distro=''):
        SrcROSBase.__init__(self, name, ros_distro)
        self.ros_deb_packages = []

    def is_debian_package(self, pkg):
        if pkg.startswith('ros-'):
            return True
        return False

    def validate(self, ros_pkgs):
        for pkg in comma_list_to_array(ros_pkgs):
            deb_pkg = pkg
            if not self.is_debian_package(pkg):
                deb_pkg = self.get_debian_ros_package_name(pkg)
            self.ros_deb_packages.append(deb_pkg)

    def get_deb_package_names(self, ros_pkgs):
        return self.ros_deb_packages



#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

from auto_abi_checker.utils import _check_call, error, info
from auto_abi_checker.srcs_base import SrcBase

from os import path, listdir
from shutil import copytree


class SrcLocalDir(SrcBase):
    def __init__(self, name):
        SrcBase.__init__(self, name)

    def validate(self, directory):
        if (not path.isdir(directory)):
            error("Path " + directory + " is not a directory")
        if not listdir(directory):
            error("Directory " + directory + "is empty")

    def run(self, directory):
        self.validate(directory)
        self.copy_files(directory)

    def copy_files(self, directory):
        info("Run copytree from  " + directory + " to " + self.ws_files)
        copytree(directory, self.ws_files)



#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

from auto_abi_checker.srcs_apt import SrcOSRFPkgGenerator
from auto_abi_checker.srcs_ros import SrcROSRepoGenerator, SrcROSPkgGenerator
from auto_abi_checker.srcs_local import SrcLocalDir
from auto_abi_checker.utils import error


class SrcGenerator:
    def generate(self, src_type, name):
        if (src_type == 'ros-pkg'):
            return SrcROSPkgGenerator(name)
        if (src_type == 'ros-repo'):
            return SrcROSRepoGenerator(name)
        elif (src_type == 'osrf-pkg'):
            return SrcOSRFPkgGenerator(name)
        elif (src_type == 'local-dir'):
            return SrcLocalDir(name)
        else:
            error("Internal error SrcGenerator. Unknown src-type ")



#!/usr/bin/env python3

import os
from setuptools import setup
import unittest

install_requires = [
    'docopt',
    'rosdistro',
]

def discover_tests():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('src/auto_abi_checker/tests', pattern='*_TEST.py')
    return test_suite

kwargs = {
    'name': 'auto_abi_checker',
    'version': '0.1.1',
    'packages': ['auto_abi_checker'],
    'package_dir': {'': 'src'},
    'author': 'Jose Luis Rivero',
    'author_email': 'jrivero@osrfoundation.org',
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License'
    ],
    'description': 'Tool to facilitate the run of abi-compliance-checker',
    'long_description': 'Tool designed to facilitate the run the great ABI compliance checker by supporting easy inputs and no configuration files',
    'license': 'Apache License 2.0',
    'python_requires': '>=3.0',
    'scripts': ['src/auto_abi_checker/auto-abi.py'],
    'install_requires': install_requires,
    'url': 'https://github.com/osrf/auto-abi-checker',
    'test_suite': 'src/auto_abi_checker/test/auto-abi_TEST.py',
    'download_url': 'https://github.com/osrf/auto-abi-checker/archive/0.1.1.tar.gz',
}

setup(**kwargs)



#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

from os import chdir, environ
from auto_abi_checker.utils import _check_call, error
from auto_abi_checker.srcs_base import SrcBase


class SrcAptBase(SrcBase):
    def __init__(self, name):
        SrcBase.__init__(self, name)

    def run(self, value):
        self.validate(value)
        pkgs = self.get_deb_package_names(value)
        self.download_deb_packages(pkgs)
        self.extract_deb_files()

    # override if need validation
    def validate(self, value):
        True

    def get_deb_package_names(self, value):
        raise NotImplementedError()

    def download_deb_packages(self, package_names):
        for p in package_names:
            # run_apt_update
            chdir(self.ws)
            result = _check_call(['apt-get', 'download', '-qq', p])
            if result != 0:
                error("Failed to download")

    def run_apt_update(self):
        _check_call(['apt-get', 'update'])

    def extract_deb_files(self):
        files = self.list_files('*.deb')
        for f in files:
            result = _check_call(['dpkg', '-x', f, self.ws_files])
            if result != 0:
                error("Failed to extract files")


class SrcOSRFPkgGenerator(SrcAptBase):
    def __init__(self, name):
        SrcAptBase.__init__(self, name)
        self.osrf_url_base = 'http://bitbucket.org/osrf/'
        self.compilation_flags.append('--std=c++17')

    def get_deb_package_names(self, osrf_repo):
        return ["lib" + osrf_repo, "lib" + osrf_repo + "-dev"]
